/*
 *	DashboardColumn.mqh
 *
 *	Dashboard in a separate chart panel
 *
 *	Copyright 2020, Orchard Forex
 *	https://www.orchardforex.com
 *
 */

/*
 *	Version History
 *
 *	Version 2
 *		2.1	Add checking for invalid index numbers
 *		2.2	Add corner selection and xdistance
 *
 *	Version 3
 *		3.1	Draw in a second panel
 *
 */

#property copyright "Copyright 2020, Orchard Forex"
#property link      "https://www.orchardforex.com"
#property version   "1.00"
#property strict

#include "DashboardV2.mqh"

class CDashboardColumn : public CDashboardV2 {

protected:	// Member variables

	int				mColumnNumber;			//	Just number in order

protected:	//	Functions
	
	virtual string	TextObjectPartial()				{	return(StringFormat("%s_%i_%i_", mName, mSubWindow, mColumnNumber));	}	//	Partial object name

	virtual void	DeleteAll();																							//	Delete all rows and objects

public:	// Constructors and initialisers

	//CDashboardColumn()																											//	Not using a default constructor
	CDashboardColumn(string name, int subWindow, int column, int xDistance, int yDistance);

	void				Init(int subWindow, int column);

public:		//	Properties

	int				GetSubWindow()						{	return(mSubWindow);							}
						// No set chart id, only way to change is start again
	
};

//+------------------------------------------------------------------+
//	Constructors and Initialisers
//+------------------------------------------------------------------+

CDashboardColumn::CDashboardColumn(string name, int subWindow, int column, int xDistance, int yDistance)
								:	mColumnNumber(-1) {

	Init(name, 0, xDistance, yDistance);
	Init(subWindow, column);

}

void		CDashboardColumn::Init(int subWindow, int column) {

	//
	//	Standard initialisation routine
	//

	mSubWindow					=	subWindow;			//	Also needed before we do any cleanup
	mColumnNumber			=	column;

	DeleteAll();									//	In case there are any old objects remaining on screen

}

void		CDashboardColumn::DeleteAll() {

	//	Overrides parent class
	//	This allows checking so the function isn't used before column and chart id are set
	//
	
	if (mColumnNumber<0 || mSubWindow<1) return;
	
	CDashboardV2::DeleteAll();

}


